package fr.esial.seenshare.models;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import com.google.gdata.client.photos.PicasawebService;
import com.google.gdata.data.photos.AlbumEntry;
import com.google.gdata.data.photos.AlbumFeed;
import com.google.gdata.data.photos.PhotoEntry;
import com.google.gdata.data.photos.UserFeed;
import com.google.gdata.util.AuthenticationException;
import com.google.gdata.util.NotImplementedException;
import com.google.gdata.util.ServiceException;
import fr.esial.seenshare.utils.ProjectFiles;
import fr.esial.seenshare.views.ErrorPopup;

/**
 * Class that manages an album It can be online, local or synchronized It
 * contains an AlbumEntry to be compatible with Picasa API
 */
@SuppressWarnings("serial")
public class Album extends Resource<AlbumEntry> implements Serializable {

    private ArrayList<Media> medias;
    private int onlineSize = 0;

    /**
     * Creates an empty album
     *
     * @throws URISyntaxException
     */
    public Album() throws URISyntaxException {
        super();
        medias = new ArrayList<Media>();
        entry = new AlbumEntry();
        entry.setAccess("private");
    }

    /**
     * Creates an empty album with a given name
     *
     * @param name name of the album
     * @throws URISyntaxException
     */
    public Album(String name) throws URISyntaxException {
        this();
        setTitle(name);
    }

    /**
     * Get a media in the album described by his title
     *
     * @param name title of the media
     * @return media
     */
    public Media getMediaByTitle(String name) {
        for (Media m : medias) {
            if (m.getTitle().equals(name))
                return m;
        }

        return null;
    }

    /**
     * Add a media to the album
     *
     * @param e         media to add
     * @param singleAdd indicate if you add a single media, from drag'n drop or
     *                  contextual "Add media" menu, to upload it if true (different
     *                  from batch addMedia calls)
     */
    public void addMedia(Media e, boolean singleAdd) {
        if (medias.add(e)) {

            // if online album and single media adding, upload the new media
            if (status != ResourceStatus.LOCAL && singleAdd) {
                try {
                    PicasawebService myService = GoogleService
                            .getGoogleService();

                    myService.getRequestFactory().setHeader("If-Match", null);

                    try {
                        Albums.getInstance().getSemaphore().acquire();
                    } catch (InterruptedException e1) {
                        e1.printStackTrace();
                    }
                    URL albumPostUrl = new URL(
                            "https://picasaweb.google.com/data/feed/api/user/default/albumid/"
                                    + this.getId());
                    PhotoEntry returnedPhoto = myService.insert(albumPostUrl, e
                            .getEntry());
                    e.setEntry(returnedPhoto);
                    Albums.getInstance().getSemaphore().release();

                } catch (UnknownHostException ex) {
                    ErrorPopup.showNetworkError();
                    ex.printStackTrace();
                } catch (IOException ex) {
                    ex.printStackTrace();
                } catch (NotImplementedException ex) {

                } catch (ServiceException ex) {
                    ErrorPopup.showServiceError();
                    ex.printStackTrace();
                }
            }

            sort();
            setChanged();
        }
        notifyObservers();
    }

    /**
     * Remove a media from the album
     *
     * @param e media to remove
     */
    public void removeMedia(Media e) {
        if (medias.remove(e)) {

            // if online album delete the media (for drag'n drop move)
            if (status != ResourceStatus.LOCAL) {
                try {
                    e.getEntry().delete();

                } catch (UnknownHostException ex) {
                    ErrorPopup.showNetworkError();
                } catch (IOException ex) {
                    ex.printStackTrace();
                } catch (ServiceException ex) {
                    ErrorPopup.showServiceError();
                }
            }

            setChanged();
        }
        notifyObservers();
    }

    /**
     * Get the media size
     *
     * @return media size
     */
    public int size() {
        return medias.size();
    }

    /**
     * Get all medias in the album
     *
     * @return list of medias
     */
    public ArrayList<Media> getMedias() {
        return medias;
    }

    /**
     * Sort all medias of the album by title
     */
    private void sort() {
        Collections.sort(medias, new Comparator<Media>() {
            public int compare(Media o1, Media o2) {
                return o1.getTitle().compareTo(o2.getTitle());
            }
        });
    }

    /**
     * Get the entry of the album (to match Picasa albums)
     *
     * @return the entry
     */
    public AlbumEntry getEntry() {
        return (AlbumEntry) entry;
    }

    /**
     * Set the entry of the album
     *
     * @param entry the entry to set
     */
    public void setEntry(AlbumEntry entry) {
        if (entry == null)
            this.entry = null;
        else
            this.entry = new AlbumEntry(entry);
    }

    /**
     * Get the access level to this album
     */
    public String getAccess() {
        return entry.getAccess();
    }

    /**
     * Set the access level to this album
     *
     * @param access access (private, protected or public)
     */
    public void setAccess(String access) {
        entry.setAccess(access);
        new SaveResourceThread(this).start();
    }

    /**
     * Get the Date the album was taken
     */
    public Date getDate() {
        return entry.getDate();
    }

    /**
     * Set the Date the album was taken
     */
    public void setDate(Date d) {
        entry.setDate(d);
        save();
    }

    /**
     * Synchronize an album with Picasa A local album is uploaded, an online
     * album is downloaded
     *
     * @throws IOException
     * @throws ServiceException
     * @throws URISyntaxException
     */
    public void synchronizeNewAlbum() throws IOException, ServiceException,
            URISyntaxException {

        PicasawebService myService = GoogleService.getGoogleService();
        myService.getRequestFactory().setHeader("If-Match", null);

        // if local album
        if (getStatus() == ResourceStatus.LOCAL) {
            try {
                Albums.getInstance().getSemaphore().acquire();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

            // get online albums list
            URL feedUrl = new URL(
                    "https://picasaweb.google.com/data/feed/api/user/default?kind=album");
            UserFeed myUserFeed = myService.getFeed(feedUrl, UserFeed.class);

            Albums.getInstance().getSemaphore().release();

            ArrayList<String> onlineAlbums = new ArrayList<String>();
            for (AlbumEntry myAlbum : myUserFeed.getAlbumEntries()) {
                onlineAlbums.add(myAlbum.getTitle().getPlainText());
            }
            // if album doesnt exist online
            if (!onlineAlbums.contains(getTitle())) {
                try {
                    Albums.getInstance().getSemaphore().acquire();
                } catch (InterruptedException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }

                // upload album
                URL albumPostUrl = new URL(
                        "https://picasaweb.google.com/data/feed/api/user/default");
                AlbumEntry insertedAlbum = myService.insert(albumPostUrl, this
                        .getEntry());

                Albums.getInstance().getSemaphore().release();

                this.setEntry(insertedAlbum);

                try {
                    Albums.getInstance().getSemaphore().acquire();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

                // upload medias in the album
                albumPostUrl = new URL(
                        "https://picasaweb.google.com/data/feed/api/user/default/albumid/"
                                + getId());

                // notify progress on status bar
                CurrentProgressStatus progressStatus = CurrentProgressStatus
                        .getInstance();
                progressStatus.setMessage("Uploading album...");
                progressStatus.setIncrement(100 / this.medias.size());

                for (Media m : this.medias) {
                    PhotoEntry returnedPhoto = myService.insert(albumPostUrl, m
                            .getEntry());

                    m.setEntry(returnedPhoto);
                    progressStatus.increment();
                }

                Albums.getInstance().getSemaphore().release();

                progressStatus.clearWork();
            }
        }

        // if online album
        else if (getStatus() == ResourceStatus.ONLINE) {
            // if album doesn't exist locally
            if (!ProjectFiles.folderExists(ProjectFiles.getProjectFolderName()
                    + "/" + this.getTitle())) {
                // download the album and store it in ~/SeeNShare/AlbumTitle
                String albumDir = ProjectFiles.getAlbumFolder(this.getTitle())
                        .toString();
                this.setResourcePath(albumDir);

                // notify progress on status bar
                CurrentProgressStatus progressStatus = CurrentProgressStatus
                        .getInstance();
                progressStatus.setMessage("Downloading album...");
                progressStatus.setIncrement(100 / this.medias.size());

                for (Media media : this.medias) {
                    String localFile = ProjectFiles.writeFile(media
                            .getResourcePath().toString(), albumDir + "/");
                    media.setResourcePath(localFile);
                    media.setMediaSource();

                    progressStatus.increment();
                }
                progressStatus.clearWork();
            }
        }

        // set the album status to synchronized
        setStatus(ResourceStatus.SYNCHRONIZED);
        // loop through medias to set status to synchronized
        for (Media media : this.medias) {
            media.setStatus(ResourceStatus.SYNCHRONIZED);
        }

        Albums.getInstance().refreshObservers(AlbumsMessage.SELECTION);
    }

    /**
     * Get the real online size (useful for albums being downloaded in threads)
     *
     * @return size of the album
     * @throws IOException
     * @throws ServiceException
     */
    public int getOnlineSize() throws IOException, ServiceException {
        int size = onlineSize;

        if (onlineSize == 0) {
            PicasawebService myService = GoogleService.getGoogleService();

            // Get online album
            URL feedUrl = new URL(
                    "https://picasaweb.google.com/data/feed/api/user/default/albumid/"
                            + getId());
            AlbumFeed feed = myService.getFeed(feedUrl, AlbumFeed.class);

            size = feed.getPhotoEntries().size();
        }

        return size;
    }

    /**
     * Check for synchronized albums if all values are the same locally and
     * remotely
     *
     * @throws ServiceException
     * @throws IOException
     */
    public void synchronizeWithPicasa() throws IOException, ServiceException {
        PicasawebService myService = GoogleService.getGoogleService();

        try {
            Albums.getInstance().getSemaphore().acquire();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        // Get online album
        URL feedUrl = new URL(
                "https://picasaweb.google.com/data/feed/api/user/default/albumid/"
                        + getId());
        AlbumFeed feed = myService.getFeed(feedUrl, AlbumFeed.class);
        int compareUpdated = entry.getUpdated().compareTo(feed.getUpdated());

        Albums.getInstance().getSemaphore().release();

        /*
           * Update album settings
           */
        if (compareUpdated > 0) {
            // upload local changes
            entry.update();
        } else if (compareUpdated < 0) {
            // apply remote changes
            this.entry = (AlbumEntry) feed.createEntry();
        }

        /*
           * Update media settings
           */
        List<PhotoEntry> onlinePhotoEntries = feed.getPhotoEntries();

        // Diff between local and remote
        ArrayList<String> onlinePhotoEntriesId = new ArrayList<String>();
        ArrayList<String> onlinePhotoEntriesId2 = new ArrayList<String>();
        for (PhotoEntry photo : onlinePhotoEntries) {
            onlinePhotoEntriesId.add(photo.getGphotoId());
            onlinePhotoEntriesId2.add(photo.getGphotoId());
        }
        ArrayList<String> localPhotoEntriesId = new ArrayList<String>();
        for (Media m : medias) {
            localPhotoEntriesId.add(m.getId());
        }

        onlinePhotoEntriesId.removeAll(localPhotoEntriesId);
        localPhotoEntriesId.removeAll(onlinePhotoEntriesId2);

        // Download new online media
        for (String id : onlinePhotoEntriesId) {
            //System.out.println(id);
        }
        // Upload new local media
        for (String id : localPhotoEntriesId) {
            //System.out.println(id);
        }
    }

    /**
     * Scan local folder to find newly added medias
     *
     * @throws ServiceException
     * @throws IOException
     * @throws URISyntaxException
     */
    public void synchronizeWithDisk() throws IOException, ServiceException,
            URISyntaxException {
        this.sort();
        File files[] = new File(getResourcePath().getPath()).listFiles();

        int mediaCount = 0; // <-- horrible cet algo...
        int fileCount = 0;
        while (mediaCount < medias.size() && fileCount < files.length) {
            // test correct file
            if (ProjectFiles.isValidPhotoFile(files[fileCount])
                    || ProjectFiles.isValidVideoFile(files[fileCount])) {
                int compareFiles = files[fileCount].getPath().compareTo(
                        medias.get(mediaCount).getResourcePath().getPath());
                if (compareFiles > 0) {
                    medias.get(mediaCount).delete(true);
                    mediaCount++;
                } else if (compareFiles < 0) {
                    if (ProjectFiles.isValidPhotoFile(files[fileCount]))
                        medias.add(new Photo(files[fileCount].getName(),
                                files[fileCount].getAbsolutePath()));
                    if (ProjectFiles.isValidVideoFile(files[fileCount]))
                        medias.add(new Video(files[fileCount].getName(),
                                files[fileCount].getAbsolutePath()));
                    fileCount++;
                } else {
                    mediaCount++;
                    fileCount++;
                }
            }
        }
        /**
         * TODO: scan local folders for new files To launch in a new thread See
         * http://docs.oracle.com/javase/tutorial/essential/io/notification.html
         */
    }

    /**
     * Load all online data in a thread : download all album & medias data
     *
     * @throws IOException
     * @throws ServiceException
     * @throws URISyntaxException
     */
    public synchronized void loadOnlineData() throws IOException,
            ServiceException, URISyntaxException {
        PicasawebService myService = GoogleService.getGoogleService();
        if (myService != null) {
            AlbumEntry myAlbum = getEntry();


            // Get the medias of the album
            URL feedUrl2 = new URL(
                    "https://picasaweb.google.com/data/feed/api/user/default/albumid/"
                            + myAlbum.getGphotoId() + "?imgmax=d");
            AlbumFeed feed = myService.getFeed(feedUrl2, AlbumFeed.class);

            for (PhotoEntry entry : feed.getPhotoEntries()) {
                Media m;
                if (entry.getVideoStatus() == null) {
                    m = new Photo(entry.getTitle().getPlainText(), entry
                            .getMediaContents().get(0).getUrl(), false);
                } else {
                    m = new Video(entry.getTitle().getPlainText(), entry
                            .getMediaContents().get(1).getUrl(), false);
                }

                m.setEntry(entry);
                m.downloadTags(this);
                m.setThumbnailPath(entry.getMediaThumbnails().get(0).getUrl());
                // set the media as an online media
                m.setStatus(ResourceStatus.ONLINE);

                addMedia(m, false);

                if (m instanceof Photo)
                    Albums.getInstance().loadSavedPersons(this, (Photo) m);

                if (Albums.getInstance().getSelected() == this)
                    Albums.getInstance().refreshObservers(
                            AlbumsMessage.SELECTION_AUTO);

            }
        }
    }
}
