package fr.esial.seenshare.models;

import java.awt.Point;
import java.io.*;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;
import java.util.concurrent.Semaphore;
import java.util.Observable;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.swing.tree.DefaultMutableTreeNode;

import com.google.gdata.client.photos.PicasawebService;
import com.google.gdata.data.photos.AlbumEntry;
import com.google.gdata.data.photos.TagEntry;
import com.google.gdata.data.photos.UserFeed;
import com.google.gdata.util.ServiceException;
import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.utils.ProjectFiles;
import fr.esial.seenshare.views.ErrorPopup;

/**
 * Class managing the list of all albums
 * Manages online and local photos,loading of local folders, and saving/loading it in a file
 * This is a Singleton : this can not be instantiated more than once, so constructors are private
 * Call getInstance() each time you want
 */
public class Albums extends Observable {

    // list of albums (online and locally)
    private static HashMap<String, Album> albums;
    // list of medias returned by a global search
    private ArrayList<Media> mediaSearched = null;
    // list of local folders to scan for new files :
    private ArrayList<String> folders;
    // selected album on the panel
    private String selected;
    // boolean for the connected status
    private boolean connected;
    // list of persons
    private HashMap<String, HashMap<String, Point[]>> personsBase;
    // singleton this
    private static Albums singleton = null;
    // threads manager, allows to stop
    private static ArrayList<Thread> threads = new ArrayList<Thread>();
    
    private Semaphore mutexGoogleAPI;

    /**
     * Creates an empty list of albums
     * Cannot be called directly
     */
    private Albums() {
        super();
        mutexGoogleAPI = new Semaphore(1, true);
        albums = new HashMap<String, Album>();
        selected = null;
        folders = new ArrayList<String>();
        connected = false;
    }
    
    public Semaphore getSemaphore() {
    	return mutexGoogleAPI;
    }

    /**
     * Get the working instance of albums, or create a new one
     *
     * @return singleton of albums
     */
    public synchronized static Albums getInstance() {
        if (singleton == null) {
            singleton = new Albums();
        }
        return singleton;
    }

    /**
     * Get a new instance of the singleton (useful for tests)
     *
     * @return new instance of singleton
     */
    public static Albums getNewInstance() {
        return (singleton = new Albums());
    }

    /**
     * Get the list of persons in all albums
     *
     * @return list of all persons
     */
    public HashMap<String, HashMap<String, Point[]>> getPersonsBase() {
        return personsBase;
    }

    /**
     * Search a query through all albums
     *
     * @param txt   word to search
     * @param type  type of search (all, tags, desc, title, person)
     * @param where list of albums specified
     */
    public void search(String txt, String type, String where) {
        ArrayList<Media> medias = new ArrayList<Media>();

        for (String albumName : albums.keySet()) {
            if (albumName.equalsIgnoreCase(where) || where.equalsIgnoreCase("All albums")) {
                Album a = albums.get(albumName);

                for (Media m : a.getMedias()) {
                    String compareTo = "";

                    if (type.equalsIgnoreCase("tag")) {
                        StringBuilder tags = new StringBuilder();

                        for (TagEntry t : m.getTags())
                            tags.append(t.getTitle().getPlainText() + " ");

                        compareTo = tags.toString();

                    } else if (type.equalsIgnoreCase("description")) {
                        compareTo = m.getDescription();

                    } else if (type.equalsIgnoreCase("person")) {
                        if (m instanceof Photo) {
                            StringBuilder persons = new StringBuilder();

                            for (String name : ((Photo) m).getPersons().keySet())
                                persons.append(name + " ");

                            compareTo = persons.toString();
                        }

                    } else
                        compareTo = m.getTitle();

                    if (compareTo.toLowerCase()
                            .matches(".*" + txt.toLowerCase() + ".*"))
                        medias.add(m);
                }
            }
        }

        mediaSearched = medias;

        refreshObservers(AlbumsMessage.SEARCH);
    }

    /**
     * List of medias being the result of a search
     *
     * @return list of medias in the search results
     */
    public ArrayList<Media> getMediaSearched() {
        return mediaSearched;
    }

    /**
     * Get all albums (useful to check sync status)
     *
     * @return set of albums
     */
    public HashMap<String, Album> getAlbums() {
        return albums;
    }

    /**
     * Add an album to the list of albums
     *
     * @param v album to add
     */
    public void addAlbum(Album v) {

        if (albums.get(v.getTitle()) == null) {
            albums.put(v.getTitle(), v);
            refreshObservers(AlbumsMessage.ALBUMS);
        } else {
            ErrorPopup.showAlbumExists();
        }
    }

    /**
     * Remove an album from the list
     *
     * @param k name of the album to remove
     */
    public void removeAlbum(String k, boolean confirm) {
        if (albums.get(k).delete(confirm))
            albums.remove(k);

        refreshObservers(AlbumsMessage.ALBUMS);
    }

    /**
     * Size of all albums
     *
     * @return size
     */
    public int size() {
        return albums.size();
    }

    /**
     * Get the selected album
     *
     * @return selected album
     */
    public Album getSelected() {
        if (!(selected == null)) {
            return albums.get(selected);
        }
        return null;
    }

    /**
     * Get the selected album name
     *
     * @return name name of the selected album
     */
    public String getSelectedName() {
        return selected;
    }

    /**
     * Get the selected album name
     *
     * @return name name of the selected album
     */
    public boolean isConnected() {
        return connected;
    }

    /**
     * Change the selected album
     *
     * @param selected name of the album to set as selected
     */
    public void setSelected(String selected) {
        this.selected = selected;
        this.mediaSearched = null;
        refreshObservers(AlbumsMessage.SELECTION);
    }

    /**
     * Get the tree of albums (useful for JList)
     *
     * @return tree of albums
     */
    public DefaultMutableTreeNode getTree() {
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("Albums");
        SortedSet<String> sortedset = new TreeSet<String>(albums.keySet());

        for (String name : sortedset) {
            root.add(new DefaultMutableTreeNode(name));
        }

        return root;
    }

    /**
     * Shortcut to notify observers
     *
     * @param msg message to notify
     */
    public void refreshObservers(AlbumsMessage msg) {
        setChanged();
        notifyObservers(msg);
    }

    /**
     * Load albums from the save file albums.data It contains all albums
     * serialized
     *
     * @throws IOException
     * @throws ClassNotFoundException
     */
    @SuppressWarnings("unchecked")
    public synchronized void loadAlbums() throws IOException, ClassNotFoundException {
        if (new File(Constants.DATA).exists()) {
            FileInputStream fis = new FileInputStream(Constants.DATA);
            ObjectInputStream ois = new ObjectInputStream(fis);
            albums = (HashMap<String, Album>) ois.readObject();
            ois.close();

            if ((albums != null) && !albums.isEmpty()) {
                restoreEntries();
            }
        } else
            albums = new HashMap<String, Album>();

        if (new File(Constants.DATA_PERSONS).exists()) {
            FileInputStream fis = new FileInputStream(Constants.DATA_PERSONS);
            ObjectInputStream ois = new ObjectInputStream(fis);
            personsBase = (HashMap<String, HashMap<String, Point[]>>) ois.readObject();
            ois.close();

            if (personsBase == null)
                personsBase = new HashMap<String, HashMap<String, Point[]>>();

            else {
                for (String a : albums.keySet())
                    for (Media m : albums.get(a).getMedias())
                        if (m instanceof Photo)
                            loadSavedPersons(albums.get(a), (Photo) m);
            }
        } else
            personsBase = new HashMap<String, HashMap<String, Point[]>>();

        refreshObservers(AlbumsMessage.ALBUMS);
        selected = null;
        refreshObservers(AlbumsMessage.SELECTION);
    }

    /**
     * Restore the entries of the albums from the save file albums.data
     */
    private void restoreEntries() {
        Iterator<Entry<String, Album>> iter = albums.entrySet().iterator();
        while (iter.hasNext()) {
            Entry<String, Album> pair = (Entry<String, Album>) iter.next();
            Album a = (Album) pair.getValue();

            if (a.getStatus() != ResourceStatus.ONLINE) {
                a.getSerialEntry().setEntry(a);
                for (Media m : a.getMedias()) {
                    m.getSerialEntry().setEntry(m);
                }
            }
        }
    }

    /**
     * Save the list of albums serialized in a file
     *
     * @throws IOException
     */
    @SuppressWarnings("unchecked")
    public void saveAlbums() throws IOException {
        // Copy and loop through albums not to save online albums in the file
        HashMap<String, Album> albumsDump = (HashMap<String, Album>) albums
                .clone();
        Iterator<Entry<String, Album>> iter = albumsDump.entrySet().iterator();
        while (iter.hasNext()) {
            Entry<String, Album> pair = (Entry<String, Album>) iter.next();
            Album a = (Album) pair.getValue();
            // save the entry in a serializable entry
            a.setSerialEntry(new SerializableEntry(a));
            a.setEntry(null);
            a.resetTags();

            if (a.getStatus() == ResourceStatus.ONLINE)
                iter.remove();
            else {
                for (Media m : a.getMedias()) {
                    m.bufImage = null;
                    // save the media entry in a serializable entry
                    m.setSerialEntry(new SerializableEntry(m));
                    m.setEntry(null);
                    m.resetTags();
                }
            }
        }

        FileOutputStream fos = new FileOutputStream(Constants.DATA);
        ObjectOutputStream oos = new ObjectOutputStream(fos);
        oos.writeObject(albumsDump);
        oos.close();

        // Person tags save

        for (String albumName : albums.keySet()) {
            for (Media m : albums.get(albumName).getMedias()) {
                if (m instanceof Photo && !((Photo) m).getPersons().isEmpty())
                    personsBase.put(albumName + "/" + m.getTitle(), ((Photo) m).getPersons());
            }
        }

        fos = new FileOutputStream(Constants.DATA_PERSONS);
        oos = new ObjectOutputStream(fos);
        oos.writeObject(personsBase);
        oos.close();
    }

    /**
     * Load albums remotely stored on Picasa Web
     *
     * @throws IOException
     * @throws ServiceException
     * @throws URISyntaxException
     */
    public synchronized void loadOnlineAlbums() throws IOException, ServiceException,
            URISyntaxException {
        PicasawebService myService = GoogleService.getGoogleService();
        if (myService != null) {
            URL feedUrl = new URL(
                    "https://picasaweb.google.com/data/feed/api/user/default?kind=album");
            UserFeed myUserFeed = myService.getFeed(feedUrl, UserFeed.class);

            CurrentProgressStatus status = CurrentProgressStatus.getInstance();
            status.setProgress(0);
            status.setMessage("Loading online albums…");

            List<AlbumEntry> albumEntries = myUserFeed.getAlbumEntries();
            status.setIncrement(100 / albumEntries.size());

            for (AlbumEntry myAlbum : albumEntries) {
                Album a = new Album();
                a.setEntry(myAlbum);
                // If the album doesn't exist locally or synchronized, add it to the list
                if (!albums.containsKey(a.getTitle())) {
                    a.setStatus(ResourceStatus.ONLINE);

                    new LoadOnlineAlbumsDataThread(a).start();

                    addAlbum(a);
                    status.increment();
                }
            }
            if (!GoogleService.isNull()) {
                connected = true;
                refreshObservers(AlbumsMessage.CONNECTION);
            }

            status.clearWork();
        }
    }

    /**
     * Load the list of persons saved in persons.data file
     *
     * @param album album where to search persons
     * @param photo photo where to search persons
     */
    public void loadSavedPersons(Album album, Photo photo) {
        HashMap<String, Point[]> persons;

        if ((persons = personsBase.get(album.getTitle() + "/" + photo.getTitle())) != null) {
            photo.setPersons(persons);
            personsBase.remove(album.getTitle() + "/" + photo.getTitle());
        }
    }

    /**
     * Add a local folder to the albums
     *
     * @param f name of the folder
     */
    public void addFolder(String f) {
        File folder = new File(f);
        try {
            Album album = new Album(folder.getName());
            boolean containsMedias = false;
            for (File fi : folder.listFiles()) {
                // test image file
                if (ProjectFiles.isValidPhotoFile(fi)) {
                    Photo p = new Photo(fi.getName(),
                            fi.getAbsolutePath());
                    album.addMedia(p, false);
                    containsMedias = true;
                    // test video file
                } else if (ProjectFiles.isValidVideoFile(fi)) {
                    Video v = new Video(fi.getName(),
                            fi.getAbsolutePath());
                    album.addMedia(v, false);
                    containsMedias = true;
                } else if (fi.isDirectory()) {
                    addFolder(fi.getAbsolutePath());
                }
            }
            if (containsMedias) {
                folders.add(f);
                album.setResourcePath(folder.getAbsolutePath());
                this.addAlbum(album);
            }

        } catch (URISyntaxException e) {
            e.printStackTrace();
        }
    }

    /**
     * Synchronize all albums with the disk and Picasa
     * Not really used....
     *
     * @throws IOException
     * @throws ServiceException
     * @throws URISyntaxException
     */
    public void syncAlbums() throws IOException, ServiceException,
            URISyntaxException {
        for (Album a : albums.values()) {
            /*
                * synchronize folders with disk if(a.getStatus() !=
                * ResourceStatus.ONLINE && a.getResourcePath() != null)
                * a.synchronizeWithDisk();
                */
            // synchronize synchronized albums with Picasa...
            if (a.getStatus() == ResourceStatus.SYNCHRONIZED)
                a.synchronizeWithPicasa();
        }
    }

    /**
     * Rename an album in the list
     *
     * @param old old name of the album
     * @param neu new name
     */
    public void renameAlbum(String old, String neu) {
        Album a = albums.get(old);
        a.setTitle(neu);
        albums.remove(old);
        albums.put(neu, a);
        
        refreshObservers(AlbumsMessage.ALBUMS);
    }

    /**
     * Update tags for all medias with the given id
     *
     * @param photoId id of the photo to update
     * @param tags    tags to add
     */
    public void updateTags(String photoId, String tags) {
        Set<String> albumsNames = albums.keySet();
        for (String name : albumsNames) {
            Album al = albums.get(name);
            for (Media m : al.getMedias()) {
                if (m.getId() != null && m.getId().compareTo(photoId) == 0)
                    m.setTags(tags);
            }
        }
    }

    /**
     * Remove online albums when the user disconnects
     * Stop all loading threads
     */
    public void removeOnlineAlbums() {

        ArrayList<String> keysToRemove = new ArrayList<String>();
        Set<String> albumsNames = albums.keySet();
        for (String name : albumsNames) {
            Album al = albums.get(name);
            if (al.getStatus() == ResourceStatus.ONLINE) {
                keysToRemove.add(name);
            }
        }
        for (String key : keysToRemove) {
            stopThreads();
            albums.remove(key);
        }
        refreshObservers(AlbumsMessage.ALBUMS);
    }

    /**
     * Stop all running threads from the threads manager
     */
    public static void stopThreads() {
        for (Thread t : threads) {
            t.stop();
            /*try {
                t.wait();
            } catch (InterruptedException e) {
                e.printStackTrace();
            } */
        }
    }

    /**
     * Add a running thread in the thread manager
     *
     * @param t thread to add
     */
    public static void addThread(Thread t) {
        threads.add(t);
    }

}
