package fr.esial.seenshare.models;

import java.util.Observable;

/**
 * Class to implement a progress bar, with a given message and increment
 * This is a singleton
 */
public class CurrentProgressStatus extends Observable {

    private double currentProgress;
    private String currentMessage;
    private double progressIncrement;
    private boolean indeterminate = false;
    private static CurrentProgressStatus instance;

    /**
     * Creates an initial progress bar
     */
    private CurrentProgressStatus() {
        currentProgress = -1;
        currentMessage = "";
        progressIncrement = 1;
    }

    /**
     * Get the current instance of the progress bar, or creates one
     *
     * @return singleton of the progressbar
     */
    public static CurrentProgressStatus getInstance() {
        if (instance == null)
            instance = new CurrentProgressStatus();

        return instance;
    }


    /**
     * Get current progress
     *
     * @return current progress
     */
    public double getProgress() {
        return currentProgress;
    }

    /**
     * Set the progressbar to be indeterminate
     */
    public void setIndeterminate() {
        indeterminate = true;

        setChanged();
        notifyObservers();
    }

    /**
     * Returns if the progressbar is indeterminate
     *
     * @return indeterminate status of the progressbar
     */
    public boolean isIndeterminate() {
        return indeterminate;
    }

    /**
     * Set the progress to a given value
     *
     * @param progress value of the progress to set
     */
    public void setProgress(double progress) {
        if (progress > 0)
            indeterminate = false;

        if (progress < 100)
            currentProgress = progress;
        else
            currentProgress = 100;

        setChanged();
        notifyObservers();
    }

    /**
     * Get the current increment value
     *
     * @return inc value
     */
    public double getCurrentIncrement() {
        return progressIncrement;
    }

    /**
     * Set the increment value
     *
     * @param inc increment value
     */
    public void setIncrement(double inc) {
        progressIncrement = inc;
    }

    /**
     * Increment the progressbar by increment value
     */
    public void increment() {
        this.setProgress(currentProgress + progressIncrement);
    }


    /**
     * Get the message printed on the progressbar
     *
     * @return
     */
    public String getMessage() {
        return currentMessage;
    }

    /**
     * Set the message to display on the progressbar
     *
     * @param msg
     */
    public void setMessage(String msg) {
        currentMessage = msg;
        setChanged();
        notifyObservers();
    }


    /**
     * Returns if the progressbar is already working
     *
     * @return progressbar is working
     */
    public boolean isWorking() {
        return (currentProgress >= 0);
    }

    /**
     * Reset the progressbar
     */
    public void clearWork() {
        setProgress(-1);
        setMessage("");
    }
}
