package fr.esial.seenshare.models;

import com.google.gdata.data.media.MediaFileSource;
import com.google.gdata.data.photos.PhotoEntry;
import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.utils.ImageToBufferedImage;
import fr.esial.seenshare.utils.ProjectFiles;

import javax.activation.MimetypesFileTypeMap;
import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.net.URISyntaxException;
import java.net.URL;

/**
 * Abstract class that manages all media (photos and videos) There is a lot of
 * things common to these classes, it is handled here
 * Photo Entries are created here for Picasa sync
 */
@SuppressWarnings("serial")
public abstract class Media extends Resource<PhotoEntry> implements Serializable {

    public Image bufImage;
    protected boolean modified = false;

    /**
     * Creates a new Media with a name and a path to a file
     *
     * @param name name of the media
     * @param path path to the file
     * @throws URISyntaxException
     */
    public Media(String name, String path) throws URISyntaxException {
        super(path);
        init(name, path, true);
    }

    /**
     * Creates a new Media with a name and a path to a file
     *
     * @param name    name of the media
     * @param path    path to the file
     * @param refresh refresh the media or not
     * @throws URISyntaxException
     */
    public Media(String name, String path, boolean refresh) throws URISyntaxException {
        super(path);
        init(name, path, refresh);
    }

    /**
     * Init method common to both constructors
     *
     * @param name    name of the media
     * @param path    path to the file
     * @param refresh refresh the media
     * @throws URISyntaxException
     */
    private void init(String name, String path, boolean refresh) throws URISyntaxException {
        entry = new PhotoEntry();
        entry.setClient(Constants.TITLE);

        bufImage = null;

        File file = new File(path);
        String mimeType = new MimetypesFileTypeMap().getContentType(file);
        
        entry.setMediaSource(new MediaFileSource(new File(path), mimeType));
        entry.setSize(file.length());

        setTitle(name, refresh);
    }

    /**
     * Get the entry of the media
     *
     * @return the entry
     */
    public PhotoEntry getEntry() {
        return this.entry;
    }

    /**
     * Set the entry of the media
     *
     * @param entry the entry to set
     */
    public void setEntry(PhotoEntry entry) {
        if (entry == null)
            this.entry = null;
        else
            this.entry = new PhotoEntry(entry);
    }


    /**
     * Set the media source of the media
     * It is the path to the file, useful for Picasa sync
     * Get the current resourcePath
     */
    public void setMediaSource() {
        File mediaFile = new File(getResourcePath().getPath());
        this.entry.setMediaSource(new MediaFileSource(mediaFile, "image/jpeg")); // TODO support other formats
    }

    /**
     * Set the entry media source (path to the file)
     *
     * @param mediaSource source file
     */
    public void setMediaSource(MediaFileSource mediaSource) {
        this.entry.setMediaSource(mediaSource);

        new SaveResourceThread(this, true).start();
        Albums.getInstance().refreshObservers(AlbumsMessage.SELECTION);
    }

    /**
     * Get the Image object associated with this Media
     */
    public Image getImage() {
        if (bufImage == null) {
            if (this instanceof Photo) {
                if (this.getStatus() == ResourceStatus.LOCAL
                        || this.getStatus() == ResourceStatus.SYNCHRONIZED) { // Load local image
                    bufImage = new ImageIcon(this.getResourcePath().getPath()).getImage();
                } else if (this.getStatus() == ResourceStatus.ONLINE) { // Load remote image
                    try {
                        return new ImageIcon(new URL(this.getResourcePath().toString())).getImage();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            } else if (this instanceof Video) {
                bufImage = new ImageIcon(getClass().getClassLoader().getResource(Constants.IMAGE_VIDEO)).getImage();
            }
        }
        return bufImage;
    }

    /**
     * Temporally save the image in a bugger
     *
     * @param i image to save in a buffer
     */
    public void tmpSaveImage(Image i) {
        bufImage = i;
        notifyObservers();
    }

    /**
     * Delete the buffer
     */
    public void reloadImage() {
        bufImage = null;
        notifyObservers();
    }

    /**
     * Save the buffer in a file or online
     */
    public void saveBufImage() {
        if (bufImage != null) {
            String mediaPath = "";
            if (getStatus() == ResourceStatus.ONLINE) {
                String[] paths = getResourcePath().getPath().split("/");
                String fileName = paths[paths.length - 1];
                mediaPath = System.getProperty("java.io.tmpdir") + "/" + fileName;
            } else {
                mediaPath = getResourcePath().getPath();
            }

            BufferedImage bufferedImage = ImageToBufferedImage.toBufferedImage(bufImage);
            File imageFile = new File(mediaPath);
            modified = false;

            try {
                ImageIO.write(bufferedImage, ProjectFiles.getFileExtension(mediaPath), imageFile);
            } catch (IOException e) {
                e.printStackTrace();
            }
            this.setMediaSource(new MediaFileSource(imageFile, ProjectFiles.getFileMimetype(mediaPath)));
        }
    }

    /**
     * Get if the file has been modified
     *
     * @return the file has been modified
     */
    public boolean isModified() {
        return this.modified;
    }
}
