package fr.esial.seenshare.models;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.color.ColorSpace;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.image.RescaleOp;
import java.awt.image.WritableRaster;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.io.Serializable;
import java.net.URISyntaxException;
import java.util.HashMap;

/**
 * Class that manages photos, inherits of Media
 */
public class Photo extends Media implements Serializable {

	private static final long serialVersionUID = 1L;
	public static final int CLOCKWISE = 1;
	public static final int COUNTER_CLOCKWISE = 2;

	protected HashMap<String, Point[]> persons;

	/**
	 * Construct a Photo with a name and a path to a file
	 * 
	 * @param name
	 *            name of the photo
	 * @param path
	 *            path to the photo file
	 * @throws URISyntaxException
	 */
	public Photo(String name, String path) throws URISyntaxException {
		super(name, path);
		this.persons = new HashMap<String, Point[]>();
	}

	/**
	 * Construct a Photo with a name, a path to a file, and a boolean saying if
	 * the photo has to be refreshed
	 * 
	 * @param name
	 *            name of the photo
	 * @param path
	 *            path to the file
	 * @param refresh
	 *            the photo has to be refreshed
	 * @throws URISyntaxException
	 */
	public Photo(String name, String path, boolean refresh)
			throws URISyntaxException {
		super(name, path, refresh);
		this.persons = new HashMap<String, Point[]>();
	}

	/**
	 * Get the list of persons displayed on the photo
	 * 
	 * @return list of persons displayed on the photo
	 */
	public HashMap<String, Point[]> getPersons() {
		return persons;
	}

	/**
	 * Set the list of persons displayed on the photo
	 * 
	 * @param persons
	 *            list of persons
	 */
	public void setPersons(HashMap<String, Point[]> persons) {
		this.persons = persons;
	}

	/**
	 * Rotate a picture and save it as the current media's image
	 * 
	 * @param rotateDirection
	 *            direction to rotate the image, Photo.CLOCKWISE or
	 *            Photo.COUNTER_CLOCKWISE.
	 */
	public void rotate(int rotateDirection) {
		CurrentProgressStatus status = CurrentProgressStatus.getInstance();
		status.setProgress(0);

		Image source = this.getImage();
		BufferedImage output = new BufferedImage(source.getHeight(null), source
				.getWidth(null), BufferedImage.TYPE_INT_RGB);

		Graphics2D g = output.createGraphics();

		AffineTransform origTransform = g.getTransform();
		AffineTransform rotateTransform = (AffineTransform) (origTransform
				.clone());
		double yRot = 0;
		double xRot = 0;

		status.setMessage("Rotating image...");

		if (rotateDirection == Photo.CLOCKWISE) {
			rotateTransform.rotate(Math.toRadians(90), xRot, yRot);
			g.setTransform(rotateTransform);
			g.drawImage(source, 0, -output.getWidth(), null);

			for (String n : persons.keySet()) {
				Point[] points = persons.get(n);

				points[0].setLocation(-(points[0].getY() - getImage()
						.getHeight(null) / 2), points[0].getX()
						+ getImage().getWidth(null) / 2);
				points[1].setLocation(-(points[1].getY() - getImage()
						.getHeight(null) / 2), points[1].getX()
						- getImage().getWidth(null) / 2);

				points[0].setLocation(points[0].getX()
						+ getImage().getHeight(null) / 2, points[0].getY()
						- getImage().getWidth(null) / 2);
				points[1].setLocation(points[1].getX()
						+ getImage().getHeight(null) / 2, points[1].getY()
						+ getImage().getWidth(null) / 2);
			}
		} else {
			rotateTransform.rotate(Math.toRadians(-90), xRot, yRot);
			g.setTransform(rotateTransform);
			g.drawImage(source, -output.getHeight(), 0, null);

			for (String n : persons.keySet()) {
				Point[] points = persons.get(n);

				points[0].setLocation(points[0].getY()
						+ getImage().getHeight(null) / 2,
						-(points[0].getX() - getImage().getWidth(null) / 2));
				points[1].setLocation(points[1].getY()
						- getImage().getHeight(null) / 2,
						-(points[1].getX() - getImage().getWidth(null) / 2));

				points[0].setLocation(points[0].getX()
						- getImage().getHeight(null) / 2, points[0].getY()
						+ getImage().getWidth(null) / 2);
				points[1].setLocation(points[1].getX()
						+ getImage().getHeight(null) / 2, points[1].getY()
						+ getImage().getWidth(null) / 2);
			}
		}

		g.setTransform(origTransform);

		status.setProgress(80);
		this.tmpSaveImage(output);
		status.setProgress(100);
		status.clearWork();
		g.dispose();

		modified = true;
		setChanged();
		notifyObservers();
	}

	/**
	 * Resize the picture according to the given width
	 * 
	 * @param newWidth
	 *            the desired width for the picture
	 */
	public void resize(int newWidth) {
		CurrentProgressStatus status = CurrentProgressStatus.getInstance();
		status.setProgress(0);
		status.setMessage("Resizing...");

		Image source = this.getImage();
		double oldWidth = source.getWidth(null);
		double oldHeight = source.getHeight(null);
		int newHeight = (int) ((oldHeight * newWidth) / oldWidth);

		for (String n : persons.keySet()) {
			Point[] points = persons.get(n);

			points[0].setLocation(points[0].getX() * (newWidth / oldWidth),
					points[0].getY() * (newHeight / oldHeight));

			points[1].setLocation(points[1].getX() * (newWidth / oldWidth),
					points[1].getY() * (newHeight / oldHeight));
		}

		BufferedImage output = new BufferedImage(newWidth, newHeight,
				BufferedImage.TYPE_INT_RGB);

		Graphics2D g = output.createGraphics();
		g.drawImage(source, 0, 0, newWidth, newHeight, null);

		status.setProgress(80);
		this.tmpSaveImage(output);
		status.setProgress(100);
		status.clearWork();
		g.dispose();

		modified = true;
		setChanged();
		notifyObservers();
	}

	public void tagPerson(String name, Point[] currentSel) {
		persons.put(name, currentSel.clone());
	}

	/**
	 * Crop the picture according to the given points
	 * 
	 * @param Point
	 *            points for the new image
	 */
	public void crop(Point[] currentSel) {
		CurrentProgressStatus status = CurrentProgressStatus.getInstance();
		status.setProgress(0);
		status.setMessage("Cropping...");

		Image source = this.getImage();

		persons.put("currentSel", currentSel);
		HashMap<String, Point[]> areas = new HashMap<String, Point[]>();

		// change all diagonal to "from top-left to bottom-right"
		for (String name : persons.keySet()) {
			Point[] points = persons.get(name);
			Point tmp = (Point) points[0].clone();

			// right to left
			if (points[0].getX() > points[1].getX()) {

				// top-right to bottom-left
				if (points[0].getY() < points[1].getY()) {
					points[0].setLocation(points[1].getX(), points[0].getY());
					points[1].setLocation(tmp.getX(), points[1].getY());

					// bottom-right to top-left
				} else {
					points[0] = points[1];
					points[1] = tmp;
				}

				// left to right
			} else {

				// bottom-left to top-right
				if (points[0].getY() > points[1].getY()) {
					points[0].setLocation(points[0].getX(), points[1].getY());
					points[1].setLocation(points[1].getX(), tmp.getY());
				}
			}
		}

		persons.remove("currentSel");
		status.setProgress(10);

		for (String name : persons.keySet()) {
			Point[] points = persons.get(name);

			if (!(points[1].getY() < currentSel[0].getY()
					|| points[0].getY() > currentSel[1].getY()
					|| points[1].getX() < currentSel[0].getX() || points[0]
					.getX() > currentSel[1].getX())) {

				if (points[0].getX() < currentSel[0].getX())
					points[0].setLocation(currentSel[0].getX() + 1, points[0]
							.getY());

				if (points[0].getY() < currentSel[0].getY())
					points[0].setLocation(points[0].getX(), currentSel[0]
							.getY() + 1);

				if (points[1].getX() > currentSel[1].getX())
					points[1].setLocation(currentSel[1].getX() - 1, points[1]
							.getY());

				if (points[1].getY() > currentSel[1].getY())
					points[1].setLocation(points[1].getX(), currentSel[1]
							.getY() - 1);

				points[0].setLocation(points[0].getX() - currentSel[0].getX(),
						points[0].getY() - currentSel[0].getY());
				points[1].setLocation(points[1].getX() - currentSel[0].getX(),
						points[1].getY() - currentSel[0].getY());

				areas.put(name, points);
			}
		}

		persons = areas;
		status.setProgress(20);

		int newWidth = (int) (currentSel[1].getX() - currentSel[0].getX());
		int newHeight = (int) (currentSel[1].getY() - currentSel[0].getY());

		BufferedImage output = new BufferedImage(newWidth, newHeight,
				BufferedImage.TYPE_INT_RGB);

		Graphics g = output.getGraphics();
		g.drawImage(source, 0, 0, newWidth, newHeight, (int) currentSel[0]
				.getX(), (int) currentSel[0].getY(), (int) currentSel[0].getX()
				+ newWidth, (int) currentSel[0].getY() + newHeight, null);

		status.setProgress(80);
		this.tmpSaveImage(output);
		status.setProgress(100);
		status.clearWork();
		g.dispose();

		modified = true;
		setChanged();
		notifyObservers();
	}

	/**
	 * Print the photo on the screen
	 * 
	 * @param graphics
	 *            graphics where to draw
	 * @param pageFormat
	 *            format of the page
	 * @param pageIndex
	 *            index in the page
	 * @return
	 * @throws PrinterException
	 */
	public int print(Graphics graphics, PageFormat pageFormat, int pageIndex)
			throws PrinterException {
		int retValue = Printable.NO_SUCH_PAGE;

		if (pageIndex == 0) {
			graphics.drawImage(this.getImage(), 0, 0, (int) pageFormat
					.getWidth(), (int) pageFormat.getHeight(), null, null);
		}

		return retValue;
	}

	/**
	 * Color the image to sepia
	 */
	public void toSepia() {
		toBW();
		CurrentProgressStatus status = CurrentProgressStatus.getInstance();
		status.setProgress(0);
		status.setMessage("Coloring to sepia...");

		Image source = this.getImage();
		int w = source.getWidth(null);
		int h = source.getHeight(null);

		BufferedImage output = new BufferedImage(w, h,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d = output.createGraphics();
		g2d.drawImage(source, 0, 0, w, h, null);

		int depth = 20;
		int grau = 40;
		WritableRaster raster = output.getRaster();
		// We need 3 integers (for R,G,B color values) per pixel.
		int[] pixels = new int[w * h * 3];
		raster.getPixels(0, 0, w, h, pixels);
		for (int i = 0; i < pixels.length; i += 3) {
			int r = pixels[i];
			int g = pixels[i + 1];
			int b = pixels[i + 2];
			int gry = (r + g + b) / 3;
			r = g = b = gry;
			r = r + (depth * 2);
			g = g + depth;
			if (r > 255) {
				r = 255;
			}
			if (g > 255) {
				g = 255;
			}
			if (b > 255) {
				b = 255;
			}
			b -= grau;
			if (b < 0) {
				b = 0;
			}
			if (b > 255) {
				b = 255;
			}
			pixels[i] = r;
			pixels[i + 1] = g;
			pixels[i + 2] = b;
		}
		raster.setPixels(0, 0, w, h, pixels);

		status.setProgress(80);
		this.tmpSaveImage(output);
		status.setProgress(100);
		status.clearWork();
		g2d.dispose();

		modified = true;
		setChanged();
		notifyObservers();
	}

	/**
	 * Color the image to black and white
	 */
	public void toBW() {
		CurrentProgressStatus status = CurrentProgressStatus.getInstance();
		status.setProgress(0);
		status.setMessage("Coloring to black and white...");

		Image source = this.getImage();
		int w = source.getWidth(null);
		int h = source.getHeight(null);

		BufferedImage output = new BufferedImage(w, h,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d = output.createGraphics();
		g2d.drawImage(source, 0, 0, w, h, null);

		ColorConvertOp colorConvert = new ColorConvertOp(ColorSpace
				.getInstance(ColorSpace.CS_GRAY), null);
		colorConvert.filter(output, output);

		status.setProgress(80);
		this.tmpSaveImage(output);
		status.setProgress(100);
		status.clearWork();
		g2d.dispose();

		modified = true;
		setChanged();
		notifyObservers();
	}

	/**
	 * Blur the image
	 */
	public void toBlur() {
		CurrentProgressStatus status = CurrentProgressStatus.getInstance();
		status.setProgress(0);
		status.setMessage("Bluring...");

		Image source = this.getImage();
		int w = source.getWidth(null);
		int h = source.getHeight(null);

		BufferedImage output = new BufferedImage(w, h,
				BufferedImage.TYPE_INT_RGB);
		BufferedImage output2 = new BufferedImage(w, h,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d = output.createGraphics();
		g2d.drawImage(source, 0, 0, w, h, null);

		float data[] = { 0.0625f, 0.125f, 0.0625f, 0.125f, 0.25f, 0.125f,
				0.0625f, 0.125f, 0.0625f };
		Kernel kernel = new Kernel(3, 3, data);
		ConvolveOp convolve = new ConvolveOp(kernel, ConvolveOp.EDGE_NO_OP,
				null);
		convolve.filter(output, output2);
		output = output2;

		status.setProgress(80);
		this.tmpSaveImage(output);
		status.setProgress(100);
		status.clearWork();
		g2d.dispose();

		modified = true;
		setChanged();
		notifyObservers();
	}
}
