package fr.esial.seenshare.models;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Observable;
import javax.swing.JOptionPane;

import com.google.gdata.client.Query;
import com.google.gdata.client.photos.PicasawebService;
import com.google.gdata.data.DateTime;
import com.google.gdata.data.PlainTextConstruct;
import com.google.gdata.data.TextConstruct;
import com.google.gdata.data.geo.Point;
import com.google.gdata.data.photos.AlbumEntry;
import com.google.gdata.data.photos.AlbumFeed;
import com.google.gdata.data.photos.GphotoEntry;
import com.google.gdata.data.photos.PhotoEntry;
import com.google.gdata.data.photos.TagEntry;
import com.google.gdata.util.AuthenticationException;
import com.google.gdata.util.ServiceException;
import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.views.ErrorPopup;

/**
 * A Resource is an album or a media
 * These classes have a lot infos in common
 *
 * @param <E> type of resource (Media or Album)
 */
@SuppressWarnings("serial")
public class Resource<E extends GphotoEntry<E>> extends Observable implements
        Serializable {

    private URI resourcePath;
    private URI thumbnailPath;
    protected ResourceStatus status;
    protected E entry;
    private ArrayList<TagEntry> tags;
    private SerializableEntry serialEntry;

    /**
     * Construct a new Resource
     *
     * @throws URISyntaxException
     */
    public Resource() throws URISyntaxException {
        super();
        tags = new ArrayList<TagEntry>();
        status = ResourceStatus.LOCAL;
    }

    /**
     * Construct a new resource with its path to file
     *
     * @param path
     * @throws URISyntaxException
     */
    public Resource(String path) throws URISyntaxException {
        this();
        setResourcePath(path);
    }

    /**
     * Get the tags describing the resource
     *
     * @return list of tags
     */
    public ArrayList<TagEntry> getTags() {
        return tags;
    }

    /**
     * Set the tags describing the resource
     *
     * @param tagsString tags (space separated values)
     */
    public void setTags(String tagsString) {
        ArrayList<TagEntry> tagList = new ArrayList<TagEntry>();

        for (String t : tagsString.split("\\s+")) {
            TagEntry tagEntry = new TagEntry();
            tagEntry.setTitle(new PlainTextConstruct(t));
            tagList.add(tagEntry);
        }

        if (this.tags == null || !this.tags.equals(tagList)) {
            this.tags = tagList;
            if (getStatus() != ResourceStatus.LOCAL) {
                try {
                    deleteTags();
                    saveTags();

                } catch (UnknownHostException e) {
                    ErrorPopup.showNetworkError();
                } catch (IOException e) {
                    e.printStackTrace();
                } catch (ServiceException e) {
                    ErrorPopup.showServiceError();
                }
            }
        }
    }

    /**
     * Reset the tags of the resource
     */
    public void resetTags() {
        this.tags = null;
    }

    /**
     * Save tags online
     *
     * @throws IOException
     * @throws ServiceException
     */
    private void saveTags() throws IOException, ServiceException {
        PicasawebService myService = GoogleService.getGoogleService();
        if (Albums.getInstance().getSelected() != null) {
            String albumId = Albums.getInstance().getSelected().getId();
            
            try {
				Albums.getInstance().getSemaphore().acquire();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
            
            URL feedUrl = new URL(
                    "https://picasaweb.google.com/data/feed/api/user/default/albumid/"
                            + albumId + "/photoid/" + this.getId());

            for (TagEntry tag : this.getTags())
                myService.insert(feedUrl, tag);
            
            Albums.getInstance().getSemaphore().release();
        }
    }

    /**
     * Delete all resource tags, to add them better after...
     *
     * @throws IOException
     * @throws ServiceException
     */
    private void deleteTags() throws IOException, ServiceException {
        PicasawebService myService = GoogleService.getGoogleService();
        if (Albums.getInstance().getSelected() != null) {
            String albumId = Albums.getInstance().getSelected().getId();

            try {
				Albums.getInstance().getSemaphore().acquire();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

            URL feedUrl = new URL(
                    "https://picasaweb.google.com/data/feed/api/user/default/albumid/"
                            + albumId + "/photoid/" + this.getId()
                            + "?kind=tag");

            AlbumFeed searchResultsFeed = myService.query(new Query(feedUrl),
                    AlbumFeed.class);
            
            Albums.getInstance().getSemaphore().release();

            for (TagEntry tag : searchResultsFeed.getTagEntries()) {
                tag.delete();
            }
        }
    }

    /**
     * Download the tags associated to the resource in picasaweb
     *
     * @param a album to download
     * @throws IOException
     * @throws ServiceException
     */
    public void downloadTags(Album a) throws IOException, ServiceException {
        PicasawebService myService = GoogleService.getGoogleService();

        try {
			Albums.getInstance().getSemaphore().acquire();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
        
        URL feedUrl = null;
        if (this instanceof Album)
            feedUrl = new URL(
                    "https://picasaweb.google.com/data/feed/api/user/default/albumid/"
                            + a.getId() + "?kind=tag");
        else if (this instanceof Media)
            feedUrl = new URL(
                    "https://picasaweb.google.com/data/feed/api/user/default/albumid/"
                            + a.getId() + "/photoid/" + this.getId()
                            + "?kind=tag");

        AlbumFeed searchResultsFeed = myService.query(new Query(feedUrl),
                AlbumFeed.class);
        
        for (TagEntry tag : searchResultsFeed.getTagEntries()) {
            this.tags.add(tag);
        }
        
        Albums.getInstance().getSemaphore().release();
    }

    /**
     * Get the title of the resource
     *
     * @return title
     */
    public String getTitle() {
        if (entry == null || entry.getTitle() == null)
            return "";
        return entry.getTitle().getPlainText();
    }

    /**
     * Set the title of the resource
     *
     * @param t title
     */
    public void setTitle(String t) {
        setTitle(t, true);
    }

    /**
     * Get the entry of the resource
     *
     * @return the entry
     */
    public SerializableEntry getSerialEntry() {
        return serialEntry;
    }

    /**
     * Set the entry of the resource
     *
     * @param serialEntry entry to set
     */
    public void setSerialEntry(SerializableEntry serialEntry) {
        this.serialEntry = serialEntry;
    }

    /**
     * Set the title of the resource
     *
     * @param t title
     */
    public void setTitle(String t, boolean refresh) {
        if (getTitle().compareTo(t) != 0) {
            entry.setTitle(TextConstruct.plainText(t));

            new SaveResourceThread(this).start();

            if (refresh) {
                if (this instanceof Media
                        && Albums.getInstance().getSelected() != null
                        && refresh)
                    Albums.getInstance().refreshObservers(
                            AlbumsMessage.SELECTION);

                if (this instanceof Album)
                    Albums.getInstance().refreshObservers(AlbumsMessage.ALBUMS);
            }
        }
    }

    /**
     * Get the path to the resource
     *
     * @return path to the resource
     */
    public URI getResourcePath() {
        return resourcePath;
    }

    /**
     * Set the path to the resource
     *
     * @param path path to the resource
     * @throws URISyntaxException
     */
    public void setResourcePath(String path) throws URISyntaxException {
        resourcePath = new URI(null, null, path, null);
        setChanged();
        notifyObservers();
    }

    /**
     * Get the path to the thumbnail of the resource
     *
     * @return path to thumbnail
     */
    public URI getThumbnailPath() {
        return thumbnailPath;
    }

    /**
     * Set the path to the thumbnail
     *
     * @param thumbnailPath path to thumbnail
     * @throws URISyntaxException
     */
    public void setThumbnailPath(String thumbnailPath)
            throws URISyntaxException {
        this.thumbnailPath = new URI(thumbnailPath);
        setChanged();
        notifyObservers();
    }

    /**
     * Get the description of the resource
     *
     * @return description
     */
    public String getDescription() {
        if (entry.getDescription() != null)
            return entry.getDescription().getPlainText();
        return null;
    }

    /**
     * Set the description of the resource
     *
     * @param description description to set
     */
    public void setDescription(String description) {
        if (this.getDescription().compareTo(description) != 0) {
            entry.setDescription(TextConstruct.plainText(description));
            new SaveResourceThread(this).start();
            setChanged();
            notifyObservers();
        }
    }

    /**
     * Get the status of the resource (local, online or sync)
     *
     * @return the status
     */
    public ResourceStatus getStatus() {
        return status;
    }

    /**
     * Set the status of the resource (local, online or sync)
     *
     * @param status the status to set
     */
    public void setStatus(ResourceStatus status) {
        this.status = status;
        setChanged();
        notifyObservers();
    }

    /**
     * Get the unique id of the resource Useful for Picasa
     *
     * @return id
     */
    public String getId() {
        return entry.getGphotoId();
    }

    /**
     * Set the unique id of the resource
     *
     * @param id
     */
    public void setId(String id) {
        entry.setGphotoId(id);
        setChanged();
        notifyObservers();
    }

    /**
     * Get the entry corresponding to this resource Useful for Picasa sync and
     * required data
     *
     * @return
     */
    public E getEntry() {
        return entry;
    }

    /**
     * Set the entry corresponding to this resource Useful for Picasa sync and
     * required data
     *
     * @param entry
     */
    public void setEntry(E entry) {
        this.entry = entry;
        setChanged();
        notifyObservers();
    }

    /**
     * Get the geolocation point of the photo
     *
     * @return location
     */
    public Point getGeoLocation() {
        if (this instanceof Album)
            return ((AlbumEntry) entry).getGeoLocation();
        else if (this instanceof Media)
            return ((PhotoEntry) entry).getGeoLocation();
        else
            return null;
    }

    /**
     * Set the location of the photo
     *
     * @param lat latitude
     * @param lon longitude
     */
    public void setGeoLocation(Double lat, Double lon) {
        if (this instanceof Album)
            ((AlbumEntry) entry).setGeoLocation(lat, lon);
        else if (this instanceof Media)
            ((PhotoEntry) entry).setGeoLocation(lat, lon);
        new SaveResourceThread(this).start();
    }

    private static void delete(File f) {
        if (f.isDirectory()) {
            for (File fi : f.listFiles()) {
                delete(fi);
            }
        }
        f.delete();
    }

    /**
     * Delete the resource (locally and/or online)
     *
     * @throws IOException
     * @throws ServiceException
     */
    public boolean delete(boolean confirm) {
        // confirm with dialog
        int confirmDelete = !confirm ? JOptionPane.OK_OPTION : JOptionPane.showConfirmDialog(null,
                "Delete the resource from See&Share?");
        if (confirmDelete == JOptionPane.OK_OPTION) {

            // Delete from disk
            if (status != ResourceStatus.ONLINE) {
                // ask if delete from disk ?
                int confirmDeleteLocally = Constants.JUNIT ? JOptionPane.YES_OPTION : JOptionPane
                        .showConfirmDialog(null,
                                "Delete the resource from disk?");
                if (confirmDeleteLocally == JOptionPane.YES_OPTION) {
                    File f = new File(getResourcePath().getPath());
                    if (f != null && f.exists()) {
                        delete(f);
                    }
                }
            }

            // Delete from Picasa Web
            if (status != ResourceStatus.LOCAL) {
                try {
                    GoogleService.getGoogleService();
                    entry.delete();

                } catch (UnknownHostException e) {
                    ErrorPopup.showNetworkError();
                } catch (IOException e) {
                    e.printStackTrace();
                } catch (ServiceException e) {
                    ErrorPopup.showServiceError();
                }
            }

            if (this instanceof Media) {
                Albums.getInstance().getSelected().removeMedia((Media) this);
                Albums.getInstance().refreshObservers(AlbumsMessage.SELECTION);
            }

            if (this instanceof Album) {
                Albums.getInstance().setSelected(null);
                Albums.getInstance().refreshObservers(AlbumsMessage.ALBUMS);
            }
        }
        
        return confirmDelete == JOptionPane.OK_OPTION;
    }

    /**
     * Set the last updated time to now If the resource is online or
     * synchronized, update online infos
     */
    public void save() {
        save(false);
    }

    public void save(boolean updateMedia) {
        entry.setUpdated(DateTime.now());
        if (this instanceof Media && Albums.getInstance().getSelected() != null) {
            Albums.getInstance().getSelected().entry.setUpdated(DateTime.now());
        }

        if (status != ResourceStatus.LOCAL) {
            try {
                PicasawebService myService = GoogleService.getGoogleService();

                try {
					Albums.getInstance().getSemaphore().acquire();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
                
                myService.getRequestFactory().setHeader("If-Match", "*");
                
                Albums.getInstance().getSemaphore().release();
                
                this.entry.setEtag(null);

                if (updateMedia) {
                    entry.updateMedia(updateMedia);
                }
                entry.update();

                setChanged();

            } catch (UnknownHostException e) {
                ErrorPopup.showNetworkError();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (ServiceException e) {
                ErrorPopup.showServiceError();
            }
        }
        notifyObservers();
    }

}
