package fr.esial.seenshare.utils;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

import javax.activation.MimetypesFileTypeMap;


/**
 * Class that manages all files related with this software
 * A folder is created in the user home folder if necessary to store downloaded online albums
 */
public class ProjectFiles {

    /**
     * Check if a folder exists locally
     *
     * @param name name of the folder to test
     * @return folder exists
     */
    public static boolean folderExists(String name) {
        File f = new File(name);
        return f.exists() && f.isDirectory();
    }

    /**
     * Get the project folder name (absolute path)
     *
     * @return name of the folder
     */
    public static String getProjectFolderName() {
        return System.getProperty("user.home") + "/" + Constants.FOLDER + "/";
    }

    /**
     * Get a File object pointing to the project folder
     *
     * @return project folder
     */
    public static File getProjectFolder() {
        File homeDir = new File(getProjectFolderName());
        // create soft directory if no exists
        if (!homeDir.exists() || !homeDir.isDirectory()) {
            homeDir.mkdir();
        }
        return homeDir;
    }

    /**
     * Get a File object pointing to the folder of an album
     *
     * @param folderName name of the album
     * @return album folder
     */
    public static File getAlbumFolder(String folderName) {
        File albumDir = new File(getProjectFolder().getPath() + "/" + folderName);
        if (!albumDir.exists() || !albumDir.isDirectory()) {
            albumDir.mkdir();
        }
        return albumDir;
    }

    /**
     * Download a remote file and write it locally
     *
     * @param fileUrl url of the remote file
     * @param path    path where to write the file
     * @return output file name
     * @throws IOException
     */
    public static String writeFile(String fileUrl, String path)
            throws IOException {

        URL u = new URL(fileUrl);
        URLConnection uc = u.openConnection();
        int contentLength = uc.getContentLength();
        InputStream raw = uc.getInputStream();
        InputStream in = new BufferedInputStream(raw);
        byte[] data = new byte[contentLength];
        int bytesRead = 0;
        int offset = 0;
        while (offset < contentLength) {
            bytesRead = in.read(data, offset, data.length - offset);
            if (bytesRead == -1)
                break;
            offset += bytesRead;
        }
        in.close();

        if (offset != contentLength) {
            throw new IOException("Only read " + offset + " bytes; Expected "
                    + contentLength + " bytes");
        }

        String[] fileName = u.getFile().split("/");
        FileOutputStream out = new FileOutputStream(path
                + fileName[fileName.length - 1]);
        out.write(data);
        out.flush();
        out.close();

        return path + fileName[fileName.length - 1];
    }

    /**
     * Check if a file is a valid photo file with mimetype
     *
     * @param f file to check
     * @return valid photo
     */
    public static boolean isValidPhotoFile(File f) {
        String type = new MimetypesFileTypeMap().getContentType(f);
        String ex = getFileExtension(f.getAbsolutePath());
        return f.isFile()
                && (type.equals("image/jpeg") || ex.equals("jpg")
                || type.equals("image/bmp") || ex.equals("bmp")
                || type.equals("image/gif")  || ex.equals("gif")
                || type.equals("image/png")  || ex.equals("png"));
    }

    /**
     * Check if a file is a valid video file with mimetype
     *
     * @param f file to check
     * @return valid video
     */
    public static boolean isValidVideoFile(File f) {
        String type = new MimetypesFileTypeMap().getContentType(f);
        String ex = getFileExtension(f.getPath());
        System.out.println(f);
        return f.isFile()
                && (type.equals("video/3gpp") || ex.equals("3gp")
                || type.equals("video/avi") || ex.equals("avi")
                || type.equals("video/quicktime") || ex.equals("mov")
                || type.equals("video/mp4") || ex.equals("mp4")
                || type.equals("video/mpeg") || ex.equals("mpeg")
                || type.equals("video/mpeg4")
                || type.equals("video/msvideo")
                || type.equals("video/x-ms-asf") || ex.equals("asf")
                || type.equals("video/x-ms-wmv") || ex.equals("wmv")
                || type.equals("video/x-msvideo"));
    }

    public static String getFileMimetype(String fileName) {
        String mimeType = "";
        String ex = getFileExtension(fileName);

        if (!ex.isEmpty()) {

            if (ex.equals("jpg") || ex.equals("jpeg")) {
                mimeType = "image/jpeg";

            } else if (ex.equals("bmp")) {
                mimeType = "image/bmp";

            } else if (ex.equals("gif")) {
                mimeType = "image/gif";

            } else if (ex.equals("png")) {
                mimeType = "image/png";

            } else if (ex.equals("3gpp")) {
                mimeType = "video/3gpp";

            } else if (ex.equals("avi")) {
                mimeType = "video/avi";

            } else if (ex.equals("mp4")) {
                mimeType = "video/mp4";

            } else if (ex.equals("wmv")) {
                mimeType = "video/x-ms-wmv";
            }
        }


        return mimeType;
    }

    public static String getFileExtension(String fileName) {
        String ext = "";
        String[] paths = fileName.split("\\.");

        if (paths.length != 0) {
            ext = paths[paths.length - 1].toLowerCase();
        }
        return ext;
    }
}
