package fr.esial.seenshare.views;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.net.URISyntaxException;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.DropMode;
import javax.swing.JTree;
import javax.swing.TransferHandler;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;

import fr.esial.seenshare.controlers.AlbumsSelectionListener;
import fr.esial.seenshare.models.Album;
import fr.esial.seenshare.models.Albums;
import fr.esial.seenshare.models.AlbumsMessage;
import fr.esial.seenshare.models.Media;
import fr.esial.seenshare.models.Photo;
import fr.esial.seenshare.models.Video;
import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.utils.ProjectFiles;

/**
 * View that represents the list of albums in a jtree
 * It supports drag'n drop : it can receive events from medias dragged on it
 */
public class AlbumsView extends JTree implements Observer {

    private static final long serialVersionUID = 1L;

    private JPopupMenu popup;
    private Albums albums;

    /**
     * Create a new view displaying a list of albums
     *
     * @param a albums to display
     */
    public AlbumsView(Albums a) {
        super();
        albums = a;
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createTitledBorder("Albums"));

        setRootVisible(false);
        setModel(new DefaultTreeModel(albums.getTree()));
        addTreeSelectionListener(new AlbumsSelectionListener(albums));

        setCellRenderer(new DefaultTreeCellRenderer() {
            private static final long serialVersionUID = 1L;

            public Component getTreeCellRendererComponent(JTree tree,
                                                          Object value, boolean sel, boolean expanded, boolean leaf,
                                                          int row, boolean hasFocus) {

                super.getTreeCellRendererComponent(tree, value, sel,
                        expanded, leaf, row, hasFocus);

                Object nodeObj = ((DefaultMutableTreeNode) value).getUserObject();

                if (Albums.getInstance().getAlbums().containsKey((String) nodeObj)) {
                    switch (Albums.getInstance().getAlbums().get((String) nodeObj).getStatus()) {
                        case ONLINE:
                            setIcon(new ImageIcon(getClass().getClassLoader().getResource(Constants.ICON_ONLINE)));
                            break;

                        case SYNCHRONIZED:
                            if (Albums.getInstance().getAlbums().get((String) nodeObj).getResourcePath() != null)
                                setIcon(new ImageIcon(getClass().getClassLoader().getResource(Constants.ICON_PHYSYNC)));
                            else
                                setIcon(new ImageIcon(getClass().getClassLoader().getResource(Constants.ICON_LOGSYNC)));
                            break;

                        case LOCAL:
                            if (Albums.getInstance().getAlbums().get((String) nodeObj).getResourcePath() != null)
                                setIcon(new ImageIcon(getClass().getClassLoader().getResource(Constants.ICON_PHY)));
                            else
                                setIcon(new ImageIcon(getClass().getClassLoader().getResource(Constants.ICON_LOG)));
                    }

                }

                return this;
            }
        });

        popup = new JPopupMenu();

        JMenuItem mi = new JMenuItem("Delete");
        ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_DELETE));
        mi.setIcon(icon);
        mi.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                albums.removeAlbum(albums.getSelectedName(), true);
                albums.setSelected(null);
            }
        });
        popup.add(mi);

        JMenuItem mi2 = new JMenuItem("Rename");
        icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_EDIT));
        mi2.setIcon(icon);
        mi2.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                String name = JOptionPane.showInputDialog("Rename \""
                        + albums.getSelectedName() + "\" :");
                if (name != null && !name.isEmpty()) {
                    albums.renameAlbum(albums.getSelectedName(), name);
                }
            }
        });
        popup.add(mi2);

        addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    popup.show(AlbumsView.this, e.getX(), e.getY());
                }
            }
        });

        setDropMode(DropMode.ON);

        addImageDnDSupport();
    }

    /**
     * Add support for Drag'n Drop
     */
    private void addImageDnDSupport() {
        setTransferHandler(new TransferHandler() {
            private static final long serialVersionUID = 1L;

           public boolean canImport(TransferHandler.TransferSupport info) {
        	   try {
        		   return info.isDataFlavorSupported(DataFlavor.stringFlavor) || info.isDataFlavorSupported(new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType));

        	   } catch (ClassNotFoundException e) {
        		   e.printStackTrace();
        	   }

                return false;
            }

            public boolean importData(TransferHandler.TransferSupport info) {
                Media media;
                Transferable t = info.getTransferable();
                String albumToName;
                Album albumTo;
                
                if (!info.isDrop()) {
                    return false;
                }

                // internal DnD?
                try {
                    media = (Media) t.getTransferData(new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType));

                    albumToName = ((JTree) info.getComponent()).getDropLocation().getPath().toString();
                    albumToName = albumToName.substring(albumToName.lastIndexOf(" ") + 1, albumToName.length() - 1);

                    albumTo = albums.getAlbums().get(albumToName);
                    albumTo.addMedia(media, true);
                    
                // external?
                } catch (UnsupportedFlavorException e) {
                    
                	String importFileList;
                	
					try {
						importFileList = (String) t.getTransferData(DataFlavor.stringFlavor);
					} catch (Exception ee) {
						return false;
					}
                    
                    albumToName = ((JTree) info.getComponent()).getDropLocation().getPath().toString();
                    albumToName = albumToName.substring(albumToName.lastIndexOf(" ") + 1, albumToName.length() - 1);

                    albumTo = albums.getAlbums().get(albumToName);
                    String[] importFileNames = importFileList.split(System . getProperty("line.separator"));

                    for(String importFileName : importFileNames) {
                    	importFileName = importFileName.replaceAll("^[^/]*/*/", "/").replaceAll("[\r\n]+", "");
                    	String fileName = null;
                    	
                    	if(importFileName.lastIndexOf("/") > 0)
                    		fileName = importFileName.substring(importFileName.lastIndexOf("/") + 1, importFileName.length());
                    	
                    	if(new File(importFileName).isDirectory()) {
                    		Albums.getInstance().addFolder(importFileName);
                    		
                    	} else if(ProjectFiles.isValidPhotoFile(new File(importFileName))) {
                    		try {
								albumTo.addMedia(new Photo(fileName, importFileName), true);
							} catch (URISyntaxException e1) {
								return false;
							}
							
                    	} else if(ProjectFiles.isValidPhotoFile(new File(importFileName))) {
                    		try {
								albumTo.addMedia(new Video(fileName, importFileName), true);
							} catch (URISyntaxException e1) {
								return false;
							}

                    	} else {
                    		System.err.println(importFileName + ": Not supported file for DnD");
                    		return false;
                    	}
                    }

                // :(
                } catch (Exception e) {
                    return false;
                }
                
                return true;
            }
        });
    }

    public void update(Observable o, Object arg) {
        if (o != null && o instanceof Albums) {
            Albums albums = (Albums) o;
            if (arg != null && arg instanceof AlbumsMessage) {
                AlbumsMessage am = (AlbumsMessage) arg;
                if (am == AlbumsMessage.ALBUMS) {
                    setModel(new DefaultTreeModel(albums.getTree()));
                }
            }
        }
    }
}
