package fr.esial.seenshare.views;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.geom.Rectangle2D;

import javax.swing.JPanel;

import fr.esial.seenshare.models.Photo;
import fr.esial.seenshare.utils.Constants;

@SuppressWarnings("serial")
/**
 * Panel to display a picture
 */
public class PicturePanel extends JPanel {

    /*
      * TODO
      * listener on window resizing
      * move picture with arrow keys
      */

    private PhotoPanel container;
    private Image image;
    private Photo photo;
    private double x, y;
    /**
     * Current zoom level for this picture panel, represented as
     * a percentage of the picture size
     */
    protected double zoomLevel;

    private Point[] currentSel;
    
    
    /**
     * Create a new PicturePanel to integrate into the window.
     * @param f parent PhotoPanel
     * @param photo Photo object to dislay
     */
    public PicturePanel(PhotoPanel f, Photo photo) {
        this.photo = photo;
        this.image = photo.getImage();
        this.zoomLevel = 1;
        this.container = f;
    }
    
    
    /**
     * Overrided method taking care of drawing the picture into the panel,
     * according to the current position and zoom level. Also draws the
     * boxes for the people tags.
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        g = (Graphics2D) g;

        g.drawImage(image, (int) x, (int) y,
                (int) (image.getWidth(null) * zoomLevel),
                (int) (image.getHeight(null) * zoomLevel), this);

        ((Graphics2D) g).setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.35F));

        for (String name : photo.getPersons().keySet()) {
            Point[] points = photo.getPersons().get(name);

            Rectangle2D rect = new Rectangle2D.Double();

            rect.setFrameFromDiagonal(
                    zoomLevel * points[0].getX() + x,
                    zoomLevel * points[0].getY() + y,
                    zoomLevel * points[1].getX() + x,
                    zoomLevel * points[1].getY() + y
            );

            g.setColor(Color.BLUE);
            ((Graphics2D) g).draw(rect);
            ((Graphics2D) g).fill(rect);

            g.setColor(Color.WHITE);
            g.drawString(
                    name,
                    (int) (x + zoomLevel * ((points[0].getX() < points[1].getX() ? points[0].getX() : points[1].getX()) + 3)),
                    (int) (y + zoomLevel * ((points[0].getY() > points[1].getY() ? points[0].getY() : points[1].getY()) - 3))
            );
        }

        g.setColor(Color.RED);

        if (currentSel != null) {
            Rectangle2D rect = new Rectangle2D.Double();

            rect.setFrameFromDiagonal(
                    zoomLevel * currentSel[0].getX() + x,
                    zoomLevel * currentSel[0].getY() + y,
                    zoomLevel * currentSel[1].getX() + x,
                    zoomLevel * currentSel[1].getY() + y
            );

            ((Graphics2D) g).draw(rect);
            ((Graphics2D) g).fill(rect);
        }
    }
    
    
    /**
     * This method centers the picture into the current panel, according
     * to the panel dimension.
     */
    public void centerPic() {
        this.x = (int) (this.getWidth() / 2 - image.getWidth(null) / 2);
        this.y = (int) (this.getHeight() / 2 - image.getHeight(null) / 2);
        this.repaint();
    }
    
    
    /**
     * Check if the picture is positionned (dragged) correctly inside the panel,
     * i.e. if the picture stays inside the viewable area : if the picture is larger
     * than the panel, it should stay outside of the area at all time.
     */
    private void checkPositionConstraints() {

        double imageW = image.getWidth(null) * zoomLevel;
        double imageH = image.getHeight(null) * zoomLevel;

        /*
           * If the picture is smaller than the window, center it in the
           * appropriate direction OR if the picture is bigger and overflow the
           * window's size, align the picture with the window's borders
           */
        // Checking horizontally
        if (imageW <= this.getWidth()) {
            this.x = (int) (this.getWidth() / 2 - imageW / 2);
        } else {
            if (x > 0)
                x = 0;
            if (x + imageW < this.getWidth())
                x = (int) (this.getWidth() - imageW);
        }

        // Checking vertically
        if (imageH <= this.getHeight()) {
            this.y = (int) (this.getHeight() / 2 - imageH / 2);
        } else {
            if (y > 0)
                y = 0;
            if (y + imageH < this.getHeight())
                y = (int) (this.getHeight() - imageH);
        }
    }
    
    
    /**
     * According to the picture size, check if the zoom is not over the maximum
     * allowed, nor too little.
     */
    private void checkZoomContraints() {
        // Check if picture isn't too small on display (< half the window)
        if (image.getWidth(null) * zoomLevel < this.getWidth() * Constants.MIN_PICTURE_DISPLAY_SIZE)
            zoomLevel = (double) (this.getWidth() * Constants.MIN_PICTURE_DISPLAY_SIZE)
                    / (double) image.getWidth(null);
        if (image.getHeight(null) * zoomLevel < this.getHeight() * Constants.MIN_PICTURE_DISPLAY_SIZE)
            zoomLevel = (double) (this.getHeight() * Constants.MIN_PICTURE_DISPLAY_SIZE)
                    / (double) image.getHeight(null);

        // Check if zoom level is excessed
        if (zoomLevel > Constants.MAX_ZOOM_LEVEL)
            zoomLevel = Constants.MAX_ZOOM_LEVEL;
    }
    
    
    /**
     * Moves the picture on the panel.
     * @param dx Move the picture of dx pixels to the right
     * @param dy Move the picture of dy pixels to the bottom
     */
    public void move(int dx, int dy) {
        x += dx;
        y += dy;

        this.checkPositionConstraints();
        this.repaint();
    }
    
    
    /**
     * This method selects the person tag under the given coordinates.
     * @param x previous horizontal position of the mouse
     * @param y previous vertical position of the mouse
     * @param dx current horizontal position of the mouse
     * @param dy current horizontal position of the mouse
     */
    public void selectPerson(int x, int y, int dx, int dy) {
        if (x > this.x && x < this.x + image.getWidth(null) * zoomLevel 
        && y > this.y && y < this.y + image.getHeight(null) * zoomLevel 
        && dx > this.x && dx < this.x + image.getWidth(null) * zoomLevel
        && dy > this.y && dy < this.y + image.getHeight(null) * zoomLevel) {

            Point p1 = new Point((int) ((x - this.x) / zoomLevel), (int) ((y - this.y) / zoomLevel));
            Point p2 = new Point((int) ((dx - this.x) / zoomLevel), (int) ((dy - this.y) / zoomLevel));

            currentSel = new Point[]{p1, p2};
            this.repaint();
        }
    }
    
    
    /**
     * Unselect the current selected person tag, if any.
     */
    public void undoSelectPerson() {
        currentSel = null;
        this.repaint();
    }

    
    /**
     * Get the current selection
     * @return selection defined by two points forming a rectangle
     */
    public Point[] getCurrentSel() {
        return currentSel;
    }
    
    
    /**
     * Transform the current selection into a person tag, with the given name
     * @param name Name of the person being tagged
     */
    public void tagPerson(String name) {
        photo.tagPerson(name, currentSel);
    }

    
    /**
     * Crop the picture according to the current selection
     */
    public void crop() {
        photo.crop(currentSel);
        this.repaint();
    }
    
    
    /**
     * Get the person tagged at the given coordinates.
     * @param x x coordinate
     * @param y y coordinate
     * @return A string with the name of the person tagged, or null if
     * no tag is at the given coordinates.
     */
    public String getTagPersonName(int x, int y) {
        x = (int) ((x - this.x) / zoomLevel);
        y = (int) ((y - this.y) / zoomLevel);

        for (String name : photo.getPersons().keySet()) {
            Point[] points = photo.getPersons().get(name);

            if (((points[0].getX() < points[1].getX() && x > points[0].getX() && x < points[1].getX()) ||
                    (points[0].getX() > points[1].getX() && x < points[0].getX() && x > points[1].getX())
            ) && (
                    (points[0].getY() < points[1].getY() && y > points[0].getY() && y < points[1].getY()) ||
                            (points[0].getY() > points[1].getY() && y < points[0].getY() && y > points[1].getY()))) {
                return name;
            }
        }

        return null;
    }
    
    
    /**
     * Remove the tag with the given name.
     * @param name Name of the person whose tag should be removed.
     */
    public void removeTagPerson(String name) {
        photo.getPersons().remove(name);
        this.repaint();
    }
    
    
    /**
     * Fit the picture to the current display area.
     */
    public void fitToScreen() {

        if (image.getWidth(null) < this.getWidth()
                && image.getHeight(null) < this.getHeight()) {
            this.zoomOriginal();
        } else {
            double targetZoom;

            // Fit smallest ratio, width or height for screenSize/imageSize
            if ((double) this.getWidth() / (double) image.getWidth(null) <
                    (double) this.getHeight() / (double) image.getHeight(null))
                targetZoom = (double) this.getWidth()
                        / (double) image.getWidth(null);
            else
                targetZoom = (double) this.getHeight()
                        / (double) image.getHeight(null);

            toZoom(targetZoom, this.getWidth() / 2, this.getHeight() / 2);
        }
    }
    
    
    /**
     * Fit the picture so its width fits the display's width.
     */
    public void fitWidth() {
        double targetZoom = (double) this.getWidth()
                / (double) image.getWidth(null);
        toZoom(targetZoom, this.getWidth() / 2, this.getHeight() / 2);
    }
    
    
    /**
     * Adapt the zoom to display the picture in its original size.
     */
    public void zoomOriginal() {
        toZoom(1, this.getWidth() / 2, this.getHeight() / 2);
    }

    
    /**
     * Increment the zoom with the given value
     * @param dZoom Value to add to the zoom
     */
    public void zoom(int dZoom) {
        zoom(dZoom, this.getWidth() / 2, this.getHeight() / 2);
    }
    
    /**
     * Increment the zoom of dZoom increments, focusing the zoom on the given point
     * coordinates.
     * @param dZoom zoom increments to apply on the current zoom
     * @param zoomX X coordinate to zoom on/from
     * @param zoomY Y coordinate to zoom on/from
     */
    public void zoom(int dZoom, int zoomX, int zoomY) {
        double zoomInc = dZoom * Constants.SCROLL_ZOOM_INCREMENT * zoomLevel;
        toZoom(zoomLevel + zoomInc, zoomX, zoomY);
    }
    
    
    /**
     * Change the zoom to the given value, centering the zoom on the given 
     * point coordinates
     * @param targetZoom New value for the zoom
     * @param zoomX X coordinate to zoom on/from
     * @param zoomY Y coordinate to zoom on/from
     */
    private void toZoom(double targetZoom, int zoomX, int zoomY) {
        double oldX = this.x;
        double oldY = this.y;
        double oldZoom = this.zoomLevel;

        zoomLevel = targetZoom;
        this.checkZoomContraints();

        // Reposition the zoomed picture below given coordinates
        x = (int) ((this.zoomLevel / oldZoom) * (oldX - zoomX) + zoomX);
        y = (int) ((this.zoomLevel / oldZoom) * (oldY - zoomY) + zoomY);

        this.checkPositionConstraints();
        this.repaint();
        container.updateZoom();
    }
    
    
    /**
     * Update the displayed image with the given one, and reset display
     * parameters. Method used to refresh the picture after editing it.
     * @param i new Image to display instead of the current picture.
     */
    protected void updateImage(Image i) {
        this.image = i;
        this.centerPic();
        this.fitToScreen();
    }
}
